const path = require('path')
const webpack = require('webpack')
const createThemeColorReplacerPlugin = require('./config/plugin.config')
// gzip ѹ
const compressionWebpackPlugin = require('compression-webpack-plugin'); //

function resolve (dir) {
  return path.join(__dirname, dir)
}

/**
 * check production or preview(pro.loacg.com only)
 * @returns {boolean}
 */
function isProd () {
  return process.env.NODE_ENV === 'production'
}

// Ŀ
// Ĭ£ǼӦýĸĿ¼,
// 磺https://www.my-app.com/
// Ĭϣ'/'
// Ӧó·УҪָ·
// 磺https://www.foobar.com/my-app/
// ҪΪ'/my-app/'
const BASE_URL = isProd ? '/' : '/'

const assetsCDN = {
  css: [],
  // https://unpkg.com/browse/vue@2.6.10/
  js: [
    '//vuepkg.oss-cn-beijing.aliyuncs.com/vue.min.js',
    '//vuepkg.oss-cn-beijing.aliyuncs.com/vue-router.min.js',
    '//vuepkg.oss-cn-beijing.aliyuncs.com/vuex.min.js',
    '//vuepkg.oss-cn-beijing.aliyuncs.com/axios.min.js',
    '//vuepkg.oss-cn-beijing.aliyuncs.com/lodash.min.js',
    '//vuepkg.oss-cn-beijing.aliyuncs.com/echarts.min.js'
  ]
}

// webpack build externals
const prodExternals = {
  vue: 'Vue',
  'vue-router': 'VueRouter',
  vuex: 'Vuex',
  axios: 'axios',
  echarts: 'echarts',
  lodash : { 
    commonjs: "lodash",
    amd: "lodash",
    root: "_" // indicates global variable
  },
  'mockjs2': 'Mock'
}

// vue.config.js
const vueConfig = {
  runtimeCompiler: true, // ̬
  publicPath: BASE_URL,
  configureWebpack: {
    // webpack plugins
    plugins: [
      // Ignore all locale files of moment.js
      new webpack.IgnorePlugin(/^\.\/locale$/, /moment$/),
      new compressionWebpackPlugin({
          //[file] ᱻ滻ԭʼԴ[path] ᱻ滻ԭʼԴ· [query] ᱻ滻ɲѯַĬֵ "[path].gz[query]"
          filename: '[path].gz[query]', // ʾcompression-webpack-plugin@3.0.0ĻassetΪfilename
          // function(buf, callback) ַַ˵ zlib 㷨( zopfli 㷨)Ĭֵ "gzip"
          algorithm: 'gzip',
          //ƥԴᱻĬֵȫԴ
          test: new RegExp('\\.(' + ['js', 'css'].join('|') + ')$'),
          //ֻдСڸֵԴᱻλ bytesĬֵ 0
          threshold: 10240,
          //ֻѹСֵԴŻᱻĬֵ 0.8
          minRatio: 0.8
        })
    ],
    // if prod is on, add externals
    externals: isProd() ? prodExternals : {}
  },
  chainWebpack: (config) => {
    config.resolve.alias
      .set('@$', resolve('src'))

    const svgRule = config.module.rule('svg')
    svgRule.uses.clear()
    svgRule
      .oneOf('inline')
      .resourceQuery(/inline/)
      .use('vue-svg-icon-loader')
      .loader('vue-svg-icon-loader')
      .end()
      .end()
      .oneOf('external')
      .use('file-loader')
      .loader('file-loader')
      .options({
        name: 'assets/[name].[hash:8].[ext]'
      })

    // if prod is on
    // assets require on cdn
    if (isProd()) {
      config.plugin('html').tap(args => {
        args[0].cdn = assetsCDN
        return args
      })
    }
    
    // С
    if (process.env.npm_config_report) {
      config.plugin('webpack-bundle-analyzer')
      .use(require('webpack-bundle-analyzer').BundleAnalyzerPlugin)
      .end();
    }
  },

  css: {
    sourceMap: false,
    loaderOptions: {
      less: {
        modifyVars: {
          // less vars，customize ant design theme
          'primary-color': process.env.VUE_APP_THEME_COLOR,
          'link-color': process.env.VUE_APP_THEME_COLOR
          // 'border-radius-base': '4px'
        },
        // do not remove this line
        javascriptEnabled: true
      }
    }
  },

  devServer: {
    // development server port 8000
    port: 8000,
    disableHostCheck: true,
    // If you want to turn on the proxy, please remove the mockjs /src/main.jsL11
    proxy: {
      '/api': {
        target: 'http://192.168.16.105:8503/qpls-md',
        // target: 'http://p.iscm.360arrow.com/qpls-md',
        // ws: false,
        ws: true,
        changeOrigin: true,
        pathRewrite: {
          '^/api': ''
        }
      }
    }
  },

  // disable source map in production
  productionSourceMap: false,
  assetsDir: 'static',
  lintOnSave: undefined,
  // babel-loader no-ignore node_modules/*
  transpileDependencies: []
}

// preview.pro.loacg.com only do not use in your production;
if (process.env.VUE_APP_PREVIEW === 'true') {
  console.log('VUE_APP_PREVIEW', true)
  // add `ThemeColorReplacer` plugin to webpack plugins
  vueConfig.configureWebpack.plugins.push(createThemeColorReplacerPlugin())
}

module.exports = vueConfig
